/*********************************************************************
 *
 * File:         message_send.c
 *
 * Description:  Messages send module for conference engine
 *
 * Author:       Tomasz Stachowiak, NPAC
 *
 * Date:         10-15-96
 *
 *********************************************************************/
#include <stdio.h>
#include <stdlib.h>

#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <sys/time.h>

#include <netdb.h>
#include <unistd.h>

#include "global.h"

int getHostName(int id, char *host)
{
  struct hostent *hostinfo;
  
  hostinfo = gethostbyaddr(&id, sizeof(int), AF_INET);
  if(hostinfo == NULL)
    {
      return 0;
    }
  
  strcpy(host, hostinfo->h_name);
  return 1;
}

int writeAll(int fd, void* buffer, size_t len)
{
  int st;
  
  do
    {
      st = write( fd, buffer, len );
      if( st > 0 )
        {
	  len -= st;
	  buffer = ( char* )buffer + st;
        } 
      else
        {
	  if( st < 0 )
            {
	      perror( "writeAll" );
	      return -1;
            }
	  return 0;
        }
    }
  while( len );

  return 1;
}

int sendControlMessage(int sock, MsgHeader *header, char *msg, int len)
{
  char buf[MSG_HEADER_LENGTH];

  header->session = 0;

  buf[0] = header->tag;
  buf[1] = header->session;
  buf[2] = header->sender.handle;
  strcpy(&buf[3], header->sender.user);
  memcpy(&buf[23], &header->sender.host, sizeof(int));
  buf[27] = header->receiver.handle;
  strcpy(&buf[28], header->receiver.user);
  memcpy(&buf[48], &header->receiver.host, sizeof(int));
  memcpy(&buf[52], &len, sizeof(int));

  if(sock >= 0)
    {
      if(writeAll(sock, buf, MSG_HEADER_LENGTH) < 0)
	{
	  fprintf(stderr, "Engine: write error\n");
	  return 0;
	}
      if(writeAll(sock, msg, len) < 0)
	{
	  fprintf(stderr, "Engine: write error\n");
	  return 0;
	}
    }
  else
    return 0;

  return 1;
}


int sendServerMessage(int socket, short tag, char *msg, int len)
{
  MsgHeader header;

  header.tag = tag;
  cuserid(header.sender.user);
  header.sender.host = gethostid();
  header.dataLength = len;
  sendControlMessage(socket, &header, msg, len);
}
